/*
Copyright 2008-2010, Paulo Vinicius Wolski Radtke (pvwradtke@gmail.com)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

/**
	Chien2D v2.0 Ator

	Data: 2008

	Histrico de alteraes:

	Data       Programador      Alterao
	========== ================ ======================================================================
	13/05/2010 Fabio Binder     Inserido campo double aux em tagAtor 

**/

#ifndef ATOR_H
#define ATOR_H

// O maior nmero de quadros permitido em um animao
#define ATOR_MAX_QUADROS		32
// O maior nmero de animaes diferentes permitidos para um personagem
#define ATOR_MAX_ANIMACOES		64
// O maior nmero de efeitos sonoros que um personagem pode possuir
#define ATOR_MAX_SONS			40
// O nmero mximo de estados na pilha
#define	ATOR_MAX_PILHA			16
// O nmero mximo de eventos na fila
#define	ATOR_MAX_FILA			32
// O nmero mximo de atores no sistema
#define ATOR_MAX_TIPOS			128
// O nmero mximo de temporizadores
#define ATOR_MAX_TEMPORIZADORES	8
// O nmero mximo de variveis autilizares
#define ATOR_MAX_AUX			8

// Enumerao para estados do ator
enum {ATOR_NASCENDO, ATOR_MORTO, ATOR_ENCERRADO, ATOR_PROG_INI};

// Enumarao para os sub-estados do estado de um ator
enum {ESTADO_INICIO, ESTADO_RODANDO, ESTADO_PROG_INI};

// Enumerao para os tipos de eventos que o engine envia
enum {
		EVT_NULO,					// Evento nulo, retornado quando a fila est vazia								->ok!
		EVT_TEMPO,					// Evento que indica que um temporizador estourou
		// Eventos de foco de tela
		EVT_FOCO_TELA,				// Indica que o ator entrou dentro do foco da cena			(programador do jogo, usa funo do engine)
		EVT_PERDE_FOCO_TELA,		// Indica que o ator saiu do foco da cena					(programador do jogo, usa funo do engine)
		// Eventos de foco do mapa
		EVT_SAIU_FORA_MAPA_BAIXO,	// Indica que o ator saiu COMPLETAMENTE fora do mapa para a parte de baixo		->ok!
		EVT_SAIU_FORA_MAPA_CIMA,	// Indica que o ator saiu COMPLETAMENTE fora do mapa para a parte de cima		->ok!
		EVT_SAIU_FORA_MAPA_ESQ,		// Indica que o ator saiu COMPLETAMENTE fora do mapa para a parte da esquerda	->ok!
		EVT_SAIU_FORA_MAPA_DIR,		// Indica que o ator saiu COMPLETAMENTE fora do mapa para a parte de direita	->ok!
		// Eventos de coliso com cenrio (gerados automaticamente)
		EVT_COLIDIU_PAREDE_ESQ,		// Indica que colidiu com uma parede  esquerda									->ok!
		EVT_COLIDIU_PAREDE_DIR,		// Indica que colidiu com uma parede  direita									->ok!
		EVT_COLIDIU_PAREDE_CIMA,	// Indica que colidiu com uma parede acima										->ok!
		EVT_COLIDIU_PAREDE_BAIXO,	// Indica que colidiu com uma parede abaixo										->ok!
		// Eventos de foco do mapa
		EVT_SAIU_FORA_MAPA,			// Indica que o ator saiu COMPLETAMENTE fora do mapa ->ok!
		EVT_ENTROU_MAPA,			// Indica que o ator entrou no mapa
		// Eventos de coliso com cenrio (gerados automaticamente)
		// Verificar o subtipo do evento
		EVT_COLIDIU_PAREDE,
		EVT_TOPO_PULO,				// Indoca que chegou ao mximo que o personagem pode pular						->ok!
		EVT_COMECOU_CAIR,			// Indica que o personagem comeou a cair										->ok!
		// Eventos de coliso com atores
		EVT_COLIDIU_PERSONAGEM,		// Indica que colidiu com um personagem						(programador do jogo)
		EVT_COLIDIU_ARMADILHA,		// Indica que colidiu com um armadilha colocada no cenrio	(programador do jogo)
		// Eventos do direcional
		EVT_PRESSIONOU_ESQ,			// Indica que o direcional da esquerda foi pressionado		(programador do jogo)
		EVT_PRESSIONOU_DIR,			// Indica que o direcional da direita foi pressionado		(programador do jogo)
		EVT_PRESSIONOU_CIMA,		// Indica que o direcional para cima foi pressionado		(programador do jogo)
		EVT_PRESSIONOU_BAIXO,		// Indica que o direcional para baixo foi pressionado		(programador do jogo)
		EVT_LIBEROU_ESQ,			// Indica que o direcional da esquerda foi liberado			(programador do jogo)
		EVT_LIBEROU_DIR,			// Indica que o direcional da direita foi liberado			(programador do jogo)
		EVT_LIBEROU_CIMA,			// Indica que o direcional para cima foi liberado			(programador do jogo)
		EVT_LIBEROU_BAIXO,			// Indica que o direcional para baixo foi liberado			(programador do jogo)
		// Eventos dos botes
		EVT_PRESSIONOU_BOTAO1,		// Indica que pressionou o boto 1							(programador do jogo)
		EVT_PRESSIONOU_BOTAO2,		// Indica que pressionou o boto 2							(programador do jogo)
		EVT_PRESSIONOU_BOTAO3,		// Indica que pressionou o boto 3							(programador do jogo)
		EVT_PRESSIONOU_BOTAO4,		// Indica que pressionou o boto 4							(programador do jogo)
		EVT_PRESSIONOU_BOTAO5,		// Indica que pressionou o boto 5							(programador do jogo)
		EVT_PRESSIONOU_BOTAO6,		// Indica que pressionou o boto 6							(programador do jogo)
		EVT_LIBEROU_BOTAO1,			// Indica que pressionou o boto 1							(programador do jogo)
		EVT_LIBEROU_BOTAO2,			// Indica que pressionou o boto 2							(programador do jogo)
		EVT_LIBEROU_BOTAO3,			// Indica que pressionou o boto 3							(programador do jogo)
		EVT_LIBEROU_BOTAO4,			// Indica que pressionou o boto 4							(programador do jogo)
		EVT_LIBEROU_BOTAO5,			// Indica que pressionou o boto 5							(programador do jogo)
		EVT_LIBEROU_BOTAO6,			// Indica que pressionou o boto 6							(programador do jogo)
		// Evento de pausa do jogo
		EVT_PRESSIONOU_PAUSA,		// Indica que pressionou o boto de pausa do jogo			(programador do jogo)
		EVT_LIBEROU_PAUSA	,		// Indica que liberou o boto de pausa do jogo			(programador do jogo)
		// Eventos de animao
		EVT_RECOMECOU_ANIMACAO,		// Indica que recomeou a animao											->ok!
		// Eventos especiais de fase
		EVT_FIM_FASE,				// Indica que acabou a fase
		EVT_CHECKPOINT,				// Indica que chegou a um checkpoint
		// Eventos especiais
		EVT_CRIA_PERSONAGEM,		// Indica para o jogo que deve criar um personagem
		EVT_PLACAR,					// Indica para o jogo que um aumento no placar do jogador
		EVT_POSICAO,				// Evento para o personagem que indica uma nova posio a um personagem
		// Incio dos eventos do jogo em si
		EVT_JOGO_PROG
};

// Subtipos de eventos
enum
{
		SUBEVT_COLIDIU_PAREDE_ESQ,				// Indica que colidiu com uma parede  esquerda									->ok!
		SUBEVT_COLIDIU_PAREDE_DIR,				// Indica que colidiu com uma parede  direita									->ok!
		SUBEVT_COLIDIU_PAREDE_CIMA,				// Indica que colidiu com uma parede acima										->ok!
		SUBEVT_COLIDIU_PAREDE_BAIXO,			// Indica que colidiu com uma parede abaixo										->ok!
		SUBEVT_COLIDIU_PERSONAGEM_CIMA,			// Indica que colidiu com um personagem						(programador do jogo)
		SUBEVT_COLIDIU_PERSONAGEM_ESQUERDA,		// Indica que colidiu com um personagem						(programador do jogo)
		SUBEVT_COLIDIU_PERSONAGEM_DIREITA,		// Indica que colidiu com um personagem						(programador do jogo)
		SUBEVT_COLIDIU_PERSONAGEM_BAIXO,		// Indica que colidiu com um personagem						(programador do jogo)
		SUBEVT_SAIU_FORA_MAPA_BAIXO,			// Indica que o ator saiu COMPLETAMENTE fora do mapa para a parte de baixo		->ok!
		SUBEVT_SAIU_FORA_MAPA_CIMA,				// Indica que o ator saiu COMPLETAMENTE fora do mapa para a parte de cima		->ok!
		SUBEVT_SAIU_FORA_MAPA_ESQ,				// Indica que o ator saiu COMPLETAMENTE fora do mapa para a parte da esquerda	->ok!
		SUBEVT_SAIU_FORA_MAPA_DIR,				// Indica que o ator saiu COMPLETAMENTE fora do mapa para a parte de direita	->ok!
		SUBEVT_FIM_FASE_VITORIA,				// Indica que o jogador venceu a fase
		SUBEVT_FIM_FASE_DERROTA					// Indicaque o jogador perdeu a fase

};

// Estrutura de dados para uma animao do personagem
typedef struct tagAnimacao
{
	// O nmero total de quadros da animao
	unsigned int numQuadros;
	// O tempo entre os quadros
	unsigned int tempo;
	// A sequncia de quadros desta animao
	unsigned int quadros[ATOR_MAX_QUADROS];
}Animacao;

// Estrutura que define o estado do personagem
typedef struct tagEstadoAtor
{
	// O cdigo estado atual
	unsigned int estado;
	// O subestado
	unsigned int subestado;
}EstadoAtor;

// A pilha de estados do personagem
typedef struct tagPilhaEstados
{
	int tamanho;
	EstadoAtor pilha[ATOR_MAX_PILHA];
}PilhaEstados;

// A estrutura do evento
typedef struct tagEvento
{
	// O tipo do evento
	unsigned int tipoEvento;
	// O subtipo, pode ser o cdigo de um personagem, por exemplo (opcional)
	unsigned int subtipo;
	// A posio aonde o evento ocorreu (opciona)
	int x, y; 
	// Um valor extra, como uma direo ou pontuao (opcional)
	int valor;
}Evento;

// A fila de eventos do personagem
typedef struct tagFilaEventos
{
	int tamanho;
	int inicio;
	Evento fila[ATOR_MAX_FILA];
}FilaEventos;

// Estrutura para representar um ator
typedef struct tagAtor
{
	// O tipo do ator
	unsigned int tipo;
	// coordenadas x,y do ator
	double x, y;
	// Nmero de vidas
	unsigned int vidas;
	// Energia (se for o caso)
	unsigned int energia;
	// A velocidade do personagem
	double velocidade;
	// a direcao na qual o personagem se desloca
	int direcao;
	// direcao para a qual o personagem olha
	int olhandoPara;
	// O cdigo da animao (posio no vetor)
	unsigned int numAnimacao;
	// informao da animao atual
	int quadroAtual, tempoQuadro;
	// Os temporizadores do personagem
	unsigned long int temporizadores[ATOR_MAX_TEMPORIZADORES];
	// AS variveis auxiliares inteiras
	int aux_int[ATOR_MAX_AUX];
	// As variveis auxiliares reais
	double aux_real[ATOR_MAX_AUX];
	// Indica se o personagem est invulnervel ou no
	bool invulneravel;
	// Indica se o personagem  sujeito  gravidade (default) ou no
	bool gravidade;
	// Indica se o personagem est limitado pelo cenrio (default) ou no
	bool solido;
	// O impulso que o personagem recebe 
	double impulsoVertical;
	// O tempo de uma queda em quadros
	int quadroQueda;
	// Indica se est em foco na tela ou no
	bool naTela;

	// RESERVADO: O estado atual do ator
	EstadoAtor estado;
	// RESERVADO: Fila de eventos do personagem
	FilaEventos fila;
	// RESERVADO: pilha de estados do personagem
	PilhaEstados pilha;

	// Liberado para utilizao pelo programador, no utilizado pela Chien
	double aux;
}Ator;

// Estrutura de dados para os dados de um ator
typedef struct tagDadosAtor
{
	// As animaes do personagem
	Animacao animacoes[ATOR_MAX_ANIMACOES];
	// O vetor de efeitos sonoros
	unsigned int sons[ATOR_MAX_SONS];
	// O nmero de sons guardados
	unsigned int numSons;
	// O spriteset do personagem. Se zero, indica que o ator no foi iniciado.
	unsigned int spriteset;
	// Dimenses do sprite
	unsigned int lsprite, asprite;
	// Ponto de referncia do bounding box do ator
	int xref, yref;
	// Dimenses do bounding box do ator
	int largura, altura;
	// Vetor de pontos extremos do sprite
	int x[4], y[4];
	// Vetor de diagonais do sprite
	double diag[4];
	// Vetor de ngulos das diagonais do sprite
	double angulos[4];
	// Indica se o ator deve ter a imagem rotacionada de acordo com olhandopara
	bool rotaciona;
	// A funo que atualiza o personagem no cenrio
	bool (*atualiza)(Ator* a, unsigned int mapa);
}DadosAtor;

// Prottipos das funes

// Funo que inicia o sistema de atores. Retorna verdade se iniciou corretamente, ou
// retorna falso se no conseguir.
bool ATOR_Inicia();

// Funo que encerra o sistema de atores.
void ATOR_Encerra();

// Funo para registrar um ator no sistema. A funo recebe os dados do ator e deve 
// indicar qual o tipo (cdigo) nico. A funo retorna falso se no conseguir carregar
// os recursos, se o ator j tiver sido registrado ou se exceder o limite de atores 
// carregados.
// Um dado importante  a funo que atualiza o personagem, que deve ser escrita pelo
// programador de acordo com a assinatura proposta.
//
bool ATOR_CarregaAtorEstatico(unsigned int tipo, char *spriteset, int lsprite, 
							  int asprite, int xref, int yref, int largura, int altura,
							  Animacao *animacoes, bool rotaciona,
							  char **sons, unsigned int numSons, 
							  bool (*atualiza)(Ator* a, unsigned int mapa),
							  int qtde_animacoes=ATOR_MAX_ANIMACOES);

// Funo para eliminar um ator no sistema. 
bool ATOR_DescarregaAtor(unsigned int tipo);

// Funo que cria um novo ator, de um tipo especificado pelo programador.
// O ator  criado na posio x,y indicadas. 
// O ator comea na posio indicada, no estado ATOR_NASCENDO, 
// velocidade 0, direo 0 e olhando para 0. Nesta situao o ator no  desenhado e o 
// cdigo do programador deve coloc-lo em um estado vlido.
//
// A funo retorna o ponteiro do ator criado. Caso a alocao falhe, retorna 0.
//
Ator* ATOR_CriaAtor(unsigned int tipo, double x, double y, int direcao);

// Funo que desloca um ator na tela/cenrio e o anima.
void ATOR_AplicaEstado(Ator *a, unsigned int mapa, unsigned int larguratela, 
						unsigned int alturatela);

// Funo que envia um evento para o personagem e o coloca na sua fila. A funo retorna
// true se conseguiu adicionar o evento e false caso contrrio.
bool ATOR_EnviaEvento(Ator *a, Evento *evt);

// Funo que pega o prximo evento do personagem. Ela recebe o ponteiro do evento que ir
// receber a cpia do evento e indica se o mesmo foi recuperado com sucesso atravs do valor
// booleano de retorno.
bool ATOR_ProximoEvento(Ator *a, Evento *evt);

// Funo que limpa a fila de eventos de um ator
void ATOR_LimpaEvento(Ator *a);

// Funo que troca o estado do ator para o novo estado, indicando se o estado atual deve ser
// preservado na pilha (true) ou no (false).
void ATOR_TrocaEstado(Ator *a, unsigned int estado, bool pilha);

// Funo que troca o estado do ator para o novo estado, indicando se o estado atual deve ser
// preservado na pilha (true) ou no (false).
void ATOR_TrocaAnimacao(Ator *a, unsigned int numAnim);

// Funo que troca o estado do ator para o topo da pilha. Caso a pilha esteja vazia, o estado
// atual no  trocado
void ATOR_DesempilhaEstado(Ator *a);

// Funo para desenhar um ator, baseado no mapa e posio inicial do desenho na tela
void ATOR_Desenha(Ator *a, unsigned int idMapa, int xtela, int ytela);

// funo para centralizar a posio do mapa em funo de um personagem
void ATOR_CentraMapa(Ator *a, unsigned int idMapa, int lJanela, int aJanela);

// Funo para atualizar um ator
bool ATOR_Atualiza(Ator *a, unsigned int mapa);

// Funo para testar a coliso entre dois personagens
bool ATOR_ColidiuAtores(Ator *a, Ator *b);
// Funo para testar a coliso de um personagem com o cenrio
bool ATOR_ColidiuBlocoCenario(Ator *a, unsigned int idMapa, unsigned int codBloco);

// Funo para tocar um efeito sonoro de um ator
void ATOR_TocaEfeito(Ator *a, unsigned int id, int posicao);
// Funo para tocar o efeito sonoro de um ator em funo de sua posio na tela
void ATOR_TocaEfeitoTela(Ator *a, unsigned int id, unsigned int mapa);

// Funo para enviar um evento para o jogo. Retorna verdadeiro se conseguiu enviar o
// evento. Falso caso contrrio
bool ATOR_EnviaEventoJogo(Evento *evt);

// Funo para pegar um evento do jogo. Se o evento foi recuperado, retorna true. Caso
// contrrio, retorna falso.
bool ATOR_ProximoEventoJogo(Evento *evt);

// Use para limpar a fila de eventos do jogo
void ATOR_LimpaEventoJogo();

// Funo para dar um impulso de mandeira correta ao jogador
void ATOR_Impulsiona(Ator *a, double impulso);

#endif
