/*
Copyright 2008-2010, Paulo Vinicius Wolski Radtke (pvwradtke@gmail.com)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

#ifndef CHIEN2D2_GL_H
#define CHIEN2D2_GL_H

/**
	Chien2D v2.0 Render OpenGL

	Data: 12/06/2008

	Histrico de alteraes:

	Data       Programador      Alterao
	========== ================ ======================================================================
	21/05/2010 Fabio Binder		Inserido o parmetro de quadros/s em C2D2_Sincroniza

**/
#include <c2d2/chien2d2.h>
#include <c2d2/chien2d2_interno.h>

// funes da Chien2D2

// Funo que inicia a Chien2D 2
bool C2D2GL_Inicia(unsigned int largura, unsigned int altura, int modoVideo, char *tituloJanela);
// Funo que encerra a Chien2D 2
void C2D2GL_Encerra();
// Funo para sincronizar o vdeo a 60ppfps
void C2D2GL_Sincroniza(Uint8 fps);
// Funo para limpar a tela
void C2D2GL_LimpaTela();
// Funo para escolher a cor de limpeza da tela
void C2D2GL_TrocaCorLimpezaTela(Uint8 r, Uint8 g, Uint8 b);
// Funo para limitar a rea de desenho da tela
void C2D2GL_LimitaAreaDesenho(int x, int y, int largura, int altura);
// Funo para eliminar o limite da rea de desenho
void C2D2GL_LiberaAreaDesenho();

// Funes para manipular sprites

// funo para carregar um sprite set na memria
unsigned int C2D2GL_CarregaSpriteSet(const char *arquivo, int largura, int altura);
// Remove um sprite set da memria
void C2D2GL_RemoveSpriteSet(unsigned int id);
// Funo para desenhar um sprite
bool C2D2GL_DesenhaSprite(unsigned int identificador, unsigned int indice, int x, int y);
// Funo para desenhar um sprite distorcido e com efeitos de cor/alpha
bool C2D2GL_DesenhaSpriteEfeito(unsigned int identificador, unsigned int indice, int x[4], int y[4], Uint8 r, Uint8 g, Uint8 b, Uint8 a);


// funes especficas para a OpenGL
// Prottipos de funes internas da verso Open GL
inline void resetaGL();
inline void alteraBlend(bool estado);
void setaTexturizacao(bool estado);
inline void modoBlend(GLenum fatorOrigem, GLenum fatorDestino);
inline void setaTextura(GLuint textura);
inline void modoWrapTextura(GLenum wrapTexturaS, GLenum wraptTexturaT);
inline void modoFiltragem(GLenum filtroMax, GLenum filtroMin);


#endif
